# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - VoteRecorder Macro

    @copyright: 2008 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.

    This is like the VotingStars macro except that votes are logged for later
    counting, and the current state of the poll is not shown during voting.
"""

from MoinMoin.util import filesys, lock
from MoinMoin import wikiutil
import MoinMoin.user
import os
try:
    import cPickle as pickle
except ImportError:
    import pickle

Dependencies = ["time"]

def get_vote_directory(page, name):
    votes_dir = page.getPagePath("poll")
    if not os.path.exists(votes_dir):
        filesys.makeDirs(votes_dir)
    return votes_dir

def get_vote_pathname(page, name):
    votes_dir = get_vote_directory(page, name)
    filename = wikiutil.quoteWikinameFS(name)
    return os.path.join(votes_dir, filename)

def get_lock_directory(page, name):
    votes_dir = get_vote_directory(page, name)
    return os.path.join(votes_dir, "__lock__")

def save_votes(page, poll_name, votes):
    vote_lock = lock.WriteLock(get_lock_directory(page, poll_name))
    vote_lock.acquire()
    try:
        _save_votes(page, poll_name, votes)
    finally:
        vote_lock.release()

def _save_votes(page, poll_name, votes):
    path = get_vote_pathname(page, poll_name)
    f = open(path, "wb")
    try:
        pickle.dump(votes, f)
    finally:
        f.close()

def get_all_votes(page, poll_name):
    vote_lock = lock.ReadLock(get_lock_directory(page, poll_name))
    vote_lock.acquire()
    try:
        return _get_all_votes(page, poll_name)
    finally:
        vote_lock.release()

def _get_all_votes(page, poll_name):
    path = get_vote_pathname(page, poll_name)
    if not os.path.exists(path):
        return {}
    f = open(path, "rb")
    try:
        voting = pickle.load(f)
        return voting
    finally:
        f.close()

def execute(macro, args):
    request = macro.request
    form = macro.form
    fmt = macro.formatter
    page = fmt.page

    output = []

    # Obtain the poll name for presentation purposes.

    args = args.split(",")
    if args:
        poll_name = args[0]
    else:
        output.append(fmt.sysmsg(on=1))
        output.append(fmt.text("Please specify a poll name!"))
        output.append(fmt.sysmsg(on=0))
        return ''.join(output)

    # Check for extra arguments.
    # NOTE: This is only adequate protection if an ACL stops untrusted users
    # NOTE: from editing the page. This should actually query the ACL in order
    # NOTE: to prevent such exploits.

    public = "public" in args[1:]

    # Get the user, if known.
    # Show a message if the user is not logged in and the poll is not public.

    if not public and not request.user.valid:
        output.append(fmt.emphasis(on=1))
        output.append(fmt.text("Please log in to vote!"))
        output.append(fmt.emphasis(on=0))
        return ''.join(output)

    username = MoinMoin.user.getUserIdentification(request)
    #username = request.user.valid and request.user.name or request.user.host()
    votes = get_all_votes(page, poll_name)

    # Handle any vote.

    if form and form.has_key("vote") and form.has_key("poll"):
        poll = form["poll"][0]
        if poll == poll_name:
            votes[username] = int(form["vote"][0])
            save_votes(page, poll_name, votes)
            output.append(fmt.emphasis(on=1))
            output.append(fmt.text("Vote submitted!"))
            output.append(fmt.emphasis(on=0))
            return ''.join(output)

    # Show the choices.

    values = {
        "poll" : poll_name, "url" : page.url(request),
        "on_image" : '/wiki/modern/img/star_on.png',
        "off_image" : '/wiki/modern/img/star_off.png'
        }
    choices = range(1, 6)

    output.append(fmt.rawHTML('<form method="get" name="%(poll)s" action="%(url)s#%(poll)s">' % values))
    output.append(fmt.rawHTML('<input type="hidden" name="poll" value="%(poll)s">' % values))

    for choice in choices:
        values["choice"] = choice
        if votes.has_key(username) and choice <= votes[username]:
            values["default_image"] = values["on_image"]
        else:
            values["default_image"] = values["off_image"]
        output.append(
            fmt.rawHTML('''
                <input type="image" name="vote" value="%(choice)s" src="%(default_image)s"
                    onmouseover="this.src='%(on_image)s'"
                    onmouseout="this.src='%(default_image)s'">
                ''' % values))

    output.append(fmt.rawHTML('</form>'))

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
